/*

  Function Bisection.h:
  implements the bisection algorithm to find the solution of a transcendental equation
  f(x) = 0.

*/

#ifndef BISECTION_H

#define BISECTION_H

#include <iostream>
#include <cmath>
#include <cstdlib>
using namespace std;


void bisection( double ( * f )( double ), double * ak, double * bk, int * numIter, double eps, int verbose )
{
  double xk, fa, fb, fx, width;

  // Check the initial interval

  fa = ( *f )( *ak );
  fb = ( *f )( *bk );
  if ( fa * fb > 0.0 )
    {
      cout << "Error! The interval [" << *ak << "," << *bk << "] does not contain a zero!" << endl;
      exit( 0 );
    }

  // Initialization

  *numIter = 0;
  width = abs( *bk - *ak );
  if ( verbose == 1 )
    {
      cout << "Iter." << "\t" << "a_k" << "\t" << "b_k" << "\t" << "x_k" << "\t" << "|b_k-a_k|" << endl;
      cout.precision(15);
    }

  // Loop for the binary search

  do
    {
      xk = 0.5 * ( *ak + *bk );
      if ( verbose == 1 ) cout << *numIter << "\t" << *ak << "\t" << *bk << "\t" << xk << "\t" << width << endl;
      fa = ( *f )( *ak );
      fb = ( *f )( *bk );
      fx = ( *f )( xk );
      if ( fx == 0.0 )    // Zero exactly in xk! Unlikely, but possible!
	{
	  *ak = xk;
	  *bk = xk;
	  width = 0.0;
	}
      else
	{
	  if ( fa * fx < 0.0 ) *bk = xk;
	  if ( fx * fb  < 0.0 ) *ak = xk;
	  width = abs( *bk - *ak );
	}
      ( *numIter )++;
    } while ( width > eps );

}

#endif
