/*

  Function Newton.h:
  implements the Newton's algorithm to find the solution of a transcendental equation
  f(x) = 0.

*/

#ifndef NEWTON_H

#define NEWTON_H

#include <iostream>
#include <cmath>
#include <cstdlib>
using namespace std;


int newton( double ( * f )( double ), double ( * fp )( double ), double * xx, double * width, int maxNumIter, double eps, int verbose )
{
  double xk, xkp1, fxk, fpxk;
  int count = 0;
  
  // Initialization

  if ( verbose == 1 )
    {
      cout << "Iter." << "\t" << "x_k" << "\t" << "x_k+1" << "\t" << "|x_k+1-x_k|" << endl;
      cout.precision(15);
    }


  // Main loop

  xk = *xx;
  do
    {
      count++;
      fxk = ( *f )( xk );
      fpxk = ( *fp )( xk );
      xkp1 = xk - fxk / fpxk;
      *width = abs( xkp1 - xk );
      if ( verbose == 1 ) cout << count << "\t" << xk << "\t" << xkp1 <<"\t" << *width << endl;
      xk = xkp1;
    } while ( ( *width > eps ) && ( count < maxNumIter ) );

  *xx = xk;
  return ( count );

}

#endif
